﻿/* *********************************************************************
* Copyright (c) 1986-2018 ICONICS, Inc.
* Contains Confidential information
* THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
* **********************************************************************/

/* *********************************************************************
* This file is subject to the terms and conditions defined in 
* 'ICONICS Extensibility Toolkit LICENSE.pdf'
* which is part of this source code package.
* **********************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

using System.Runtime.Serialization;
using System.Xml.Serialization;

using Ico.Fwx.Communication.Client;

using Ico.Workflow;
using Ico.Workflow.ComponentModel;
using Ico.Workflow.Runtime.Diagrams;

using Ico.Workflow.Activities;
using Ico.Workflow.Activities.Custom;

using Ico.Workflow.Validation;

using Ico.Workflow.Misc;
using Ico.DataFlow.Schemas;
using Ico.DataFlow;

namespace FilesInfoCustomActivity
{
    /// <summary>
    /// This class represents a custom BWX64 activity type that provides some Output, which is a dataset
    /// containing rows of files information from a particular parent directory 
    /// </summary>

    [CustomActivityEditor("FilesInfoCustomActivity.Design.MyFilesInfoActivityEditor, FilesInfoCustomActivityDesign")]  //assign editor for this activity
    [CustomActivityIcon(ActivityIcon.File)]     //somtething to display in the designer
    public class MyFilesInfoActivity : CustomActivityBase, IHasOutputActivity2 //make sure to use this as a base class
    {
        /// <summary>
        /// Override this Property to return the name of your company
        /// </summary>
        public override string CompanyName
        {
            get
            {
                return "The Charging Mirash";
            }
        }

        /// <summary>
        ///   Override this Property to return your Product Name
        /// </summary>
        public override string ProductName
        {
            get
            {
                return "Elbonai Ultra Lux";
            }
        }

        /// <summary>
        ///  Override this Property to return your Activity Version
        /// </summary>
        public override string ActivityVersion
        {
            get
            {
                return "1.0.0.1";
            }
        }

        /// <summary>
        /// Override this Property to return your Activity's Category
        /// </summary>
        public override ActivityCategory Category
        {
            get
            {
                return ActivityCategory.UserDefined;
            }
        }

        /// <summary>
        ///  Override this Property to return your Activity's type name. It will be displayed as Activity Type.
        /// </summary>
        public override string TypeName
        {
            get
            {
                return "Files Info Activity";
            }
        }

        /// <summary>
        /// Override this Property to return your Activity's type description.
        /// </summary>
        public override string TypeDescription
        {
            get
            {
                return "This activity was designed for educational purpose. At runtime it will read the content of a directory and provide some output.";
            }
        }


        //************************************************************************************************
        // ADD ANY PROPERTIES YOU WANT. They must be serializable or be an arrays of serializable types.
        //************************************************************************************************

        /// <summary>
        /// Gets, Sets Parent Directory.
        /// The value will be set by our custom configurator.
        /// </summary>
        public string ParentDir
        {
            get
            { return _parentDir; }

            set
            {
                if (_parentDir != value)
                {
                    _parentDir = value;
                    OnPropertyChanged("ParentDir");
                }
            }
        }

        private string _parentDir = "c:\\Windows";



        #region Implementation of IHasOutputActivity

        /// <summary>
        /// The expression used to manipulate the rows in the dataset, once they are retrieved.
        /// We ignore it in this simple example.
        /// </summary>
        [ExpressionSyntax]
        public string RowsFilter
        {
            get { return string.Empty; }
            set { }
        }

        private ActivityColumn[] schema =  new ActivityColumn[]
                {
                    new ActivityColumn { Index = 0, DataType = Ico.Fwx.Communication.Core.BuiltInType.String, Name = "FileName" },
                    new ActivityColumn { Index = 1, DataType = Ico.Fwx.Communication.Core.BuiltInType.String, Name = "ParentFolder" },
                    new ActivityColumn { Index = 2, DataType = Ico.Fwx.Communication.Core.BuiltInType.Boolean, Name = "IsReadOnly" },
                    new ActivityColumn { Index = 3, DataType = Ico.Fwx.Communication.Core.BuiltInType.Int64, Name = "Size" },
                    new ActivityColumn { Index = 4, DataType = Ico.Fwx.Communication.Core.BuiltInType.DateTime, Name = "Created" },
                    new ActivityColumn { Index = 5, DataType = Ico.Fwx.Communication.Core.BuiltInType.DateTime, Name = "LastAccessed" },
                    new ActivityColumn { Index = 6, DataType = Ico.Fwx.Communication.Core.BuiltInType.DateTime, Name = "LastWritten" },
                };

        /// <summary>
        /// Our read-only schema for Output dataset
        /// </summary>
        [HasElements(ErrorMessageResourceType = typeof(Properties.Resources), ErrorMessageResourceName = "ErrorSchemaRequired")]
        public ActivityColumn[] Schema
        {
            get { return schema; }
            set { }
        }

        public bool IsSchemaReadOnly
        {
            get
            {
                return true;
            }
        }

        public bool SupportsErrors
        {
            get
            {
                return false;
            }
        }

        public bool SupportsDefault
        {
            get
            {
                return false;
            }
        }

        public bool SupportsKeys
        {
            get
            {
                return false;
            }
        }

        private List<DataRow> output = new List<DataRow>();
        /// <summary>
        /// The output property contains the dataset read by the reader
        /// </summary>
        [IgnoreDataMember]
        [XmlIgnore]
        public List<DataRow> Output
        {
            get { return output; }
            set
            {
                output = value;
                OnPropertyChanged("Output");
            }
        }

        public Task<ActivityColumn[]> GetDefaultSchemaAsync(FwxAsyncClient client)
        {
            return Task.FromResult<ActivityColumn[]>(schema);
        }

        #endregion //#region Implementation of IHasOutputActivity



        /// <summary>
        /// Here is the code that executes our Activity's work. It should return one of ActivityResult values.
        /// In this example we read a directory content and fill Output DataSet.
        /// </summary>
        /// <param name="cancToken"></param>
        /// <returns></returns>
        protected override async Task<ActivityResult> DoWork(CancellationToken cancToken)
        {
            //do not do anything if current transaction was cancelled for any reasons
            if (cancToken.IsCancellationRequested)
                return ActivityResult.Cancelled;

            //does configuration data make sense?
            if (string.IsNullOrEmpty(ParentDir))
            {
                LastError = "ParentDir not set!";
                return ActivityResult.Failed;
            }

            //this would resolve global aliases, language aliases or parameters in the ParentDir, if any existed.
            string resolvedParentDir = await ResolveParameter(ParentDir);

            //configuration data make sense?
            if (!System.IO.Directory.Exists(resolvedParentDir))
            {
                LastError = string.Format("Parent Path '{0}' does not exist!", resolvedParentDir);
                return ActivityResult.Failed;
            }

            ActivityResult result = ActivityResult.Succeeded;

            //execute required work
            try
            {
                System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(resolvedParentDir);
                System.IO.FileInfo[] fileInfos = dirInfo.GetFiles();

                //creates columns collection for our dataset
                DataColumnCollection columnsCollection = new DataColumnCollection(new DataColumn[] 
                                                                                    {
                                                                                        new DataColumn(schema[0].Name, new Ico.Fwx.Communication.Core.TypeInfo(schema[0].DataType)),
                                                                                        new DataColumn(schema[1].Name, new Ico.Fwx.Communication.Core.TypeInfo(schema[1].DataType)),
                                                                                        new DataColumn(schema[2].Name, new Ico.Fwx.Communication.Core.TypeInfo(schema[2].DataType)),
                                                                                        new DataColumn(schema[3].Name, new Ico.Fwx.Communication.Core.TypeInfo(schema[3].DataType)),
                                                                                        new DataColumn(schema[4].Name, new Ico.Fwx.Communication.Core.TypeInfo(schema[4].DataType)),
                                                                                        new DataColumn(schema[5].Name, new Ico.Fwx.Communication.Core.TypeInfo(schema[5].DataType)),
                                                                                        new DataColumn(schema[6].Name, new Ico.Fwx.Communication.Core.TypeInfo(schema[6].DataType)),
                                                                                    }
                                                                                  );

                output.Clear();

                foreach (System.IO.FileInfo currFI in fileInfos)
                {
                    DateTime dtCreated = currFI.CreationTime;
                    DateTime dtLastAccessed = currFI.LastAccessTime;
                    DateTime dtLastWrite = currFI.LastWriteTime;
                    string name = currFI.Name;
                    long size = currFI.Length;
                    bool isReadOnly = currFI.IsReadOnly;

                    DataRow currFileInfoRow = columnsCollection.NewRow(name, resolvedParentDir, isReadOnly, size, dtCreated, dtLastAccessed, dtLastWrite);
                    output.Add(currFileInfoRow);
                }
            }
            catch (Exception ex)
            {
                //oops, failed miserably
                LastError = string.Format("Exception at discoverying files in '{0}': {1}", resolvedParentDir, ex.Message);
                result = ActivityResult.Failed;
            }

            return result;
        }


    }
}
